#ifndef __THE_MAZE_OF_GALIOUS_2
#define __THE_MAZE_OF_GALIOUS_2

#define TILESIZE	16
#define MAX_OBJECT_PLANES	16

#define TILECATEGORIES 256
#define NOPOSITION	-999999

#define LOCATION_NONE			-1
#define LOCATION_OBJECT			0
#define LOCATION_ROOM			1
#define LOCATION_MAP			2
#define LOCATION_GAME			3
#define LOCATION_INTERFACE		5
#define LOCATION_FOREIGN		4
#define LOCATION_APPLICATION	6

/* MOG ROOM CLASS: */ 

class ROOM {
	// The EDITOR classes can access all the MOG2 members
	friend class EDITOR;
	friend class MRoomsWnd;
	friend class MMapWnd;
	friend class RVariableWnd;
	friend class REditWnd;
	friend class RToolsWnd;
	friend class OVariableWnd;
	friend class VariableListWnd;
	friend class BRMoveDlg;
	friend class BPlaneSelectionDlg;
	friend class GameDebugWindow;
	friend class BParallaxWnd;
	
	friend class ActiveMOG2;
	friend class ActiveMAP;
	friend class ActiveROOM;
	friend class ActiveOBJECT;

	friend class PLACED_OBJECT;

	friend class ActionSCRIPT;
	friend class ConditionSCRIPT;
	friend class ValueSCRIPT;

public:
	ROOM(class MAP *m=0);
	ROOM(MAP *m,int dx,int dy);
	ROOM(ROOM &r);
	~ROOM(void);

	bool save(FILE *fp,class MOG2 *game);
	bool load(FILE *fp,MOG2 *game);

	void draw(SDL_Surface *sfc,int x,int y,int dx,int dy,int stx,int sty,float scale,bool *dplane);
	void draw_frame(SDL_Surface *sfc,int x,int y,int dx,int dy,int stx,int sty,float scale);
	int get_x(void);
	int get_y(void);
	int get_dx(void);
	int get_dy(void);

	class PLACED_OBJECT *search_ladder(int x,int y,int dx,int dy);

private:
	List<class VARIABLE> room_variables;
	MAP *map;
	int x,y;
	int dx,dy;
	class ActionSCRIPT *room_action;

	int drawing_order;	/* 0 - none */ 
						/* 1 - topdown */ 
						/* 2 - bottomup */ 
						/* 3 - leftright */ 
						/* 4 - rightleft */ 

	bool horizontal_circularity;	/* If TRUE, when an object exits the right side, it will appear in the
									   left side, and viceversa. */ 
	bool vertical_circularity;		/* ... */ 


	int COLLISION_GRID_SX,COLLISION_GRID_SY,COLLISION_GRID_TOLERANCE;	/* Parameters of the collision	*/ 
																		/* hash table.					*/ 

	/* Planes information: */ 
	int used_object_planes;
	int plane_h_speed[MAX_OBJECT_PLANES];
	int plane_v_speed[MAX_OBJECT_PLANES];
	int plane_h_align[MAX_OBJECT_PLANES];
	int plane_v_align[MAX_OBJECT_PLANES];

	/* TILES + INITIAL OBJECTS: */ 
	List<class PLACED_TILE> tiles[MAX_OBJECT_PLANES];
	List<PLACED_OBJECT> objects[MAX_OBJECT_PLANES];
};


/* MOG MAP CLASS: */ 

class MAP {
	// The EDITOR classes can access all the MOG2 members
	friend class EDITOR;
	friend class GMapWnd;
	friend class MVariableWnd;
	friend class MRoomsWnd;
	friend class MMapWnd;
	friend class OVariableWnd;
	friend class VariableListWnd;
	friend class BNetworkConfigDlg;
	friend class GameWindow;
	friend class GameDebugWindow;

	friend class NetworkMOG2Protocol;

	friend class ActiveMOG2;
	friend class ActiveMAP;
	friend class NetworkGameWindow;
	friend class MOG2;
	friend class PLACED_OBJECT;

public:
	MAP(MOG2 *game);
	MAP(char *name,MOG2 *game);
	~MAP(void);

	bool save(FILE *fp,class MOG2 *game);
	bool load(FILE *fp,class MOG2 *game);

	bool cmp(char *n);
	void CMC(int *x1,int *y1,int *x2,int *y2);
	void BiggestRoom(int *dx,int *dy);
	void BiggestUnplacedRoom(int *dx,int *dy);
	bool fit_roomp(int x,int y,int dx,int dy);
	ROOM *get_room(int x,int y);

	void draw(SDL_Surface *sfc,int x,int y,int dx,int dy,int stx,int sty,float scale);
	void draw_frame(SDL_Surface *sfc,int x,int y,int dx,int dy,int stx,int sty,float scale);

	PLACED_OBJECT *search_ladder(int x,int y,int dx,int dy);

private:
	char *name;
	MOG2 *game;
	List<VARIABLE> map_variables;
	List<ROOM> rooms;

	class ActionSCRIPT *map_action;
};



class FONTCONTAINER {
public:
	FONTCONTAINER(char *name,int size);
	~FONTCONTAINER();

	void print_left(char *text,int r,int g,int b,SDL_Surface *sfc,int x,int y,int spacing);
	void print_centered(char *text,int r,int g,int b,SDL_Surface *sfc,int x,int y,int spacing);
	int get_text_width(char *text,int spacing);

	SDL_Surface *multiline_text_surface(char *text,int line_dist,SDL_Color c,int spacing);
	SDL_Surface *multiline_text_surface_limited(char *text,int line_dist,SDL_Color c,int spacing,int max_w);

	TTF_Font *ttf_font;
	SDL_Surface *bitmap_font;
	char *name;
	int size;
};

/* MAZE OF GALIOUS 2 GAME CLASS: */ 

class MOG2 {
	// The EDITOR classes can access all the MOG2 members
	friend class EDITOR;	
	friend class GVariableWnd;
	friend class AVariableWnd;
	friend class GMapWnd;
	friend class RObjectWnd;
	friend class OBitmapWnd;
	friend class OSampleWnd;
	friend class OObjectWnd;
	friend class GrabGrabberWnd;
	friend class BGamePreviewDlg;
	friend class OVariableWnd;
	friend class VariableListWnd;
	friend class BViewActionScriptWnd;
	friend class BViewValueScriptWnd;
	friend class BNetworkConfigDlg;
	friend class GameWindow;
	friend class GameDebugWindow;
	friend class SampleSelectorDlg;
	friend class IObjectWnd;
	friend class IToolsWnd;

	friend class TILE;
	friend class OBJECT;
	friend class PLACED_TILE;
	friend class PLACED_OBJECT;
	friend class ROOM;

	friend class ActiveMOG2Application;
	friend class ActiveMOG2;
	friend class ActiveROOM;
	friend class ActiveOBJECT;
	friend class ActiveINTERFACE;

	friend class ActionSCRIPT;
	friend class ValueSCRIPT;
	friend class AnimSCRIPT;

public:
	MOG2(void);
	~MOG2(void);

	bool save(FILE *fp);
	bool load(FILE *fp);

	OBJECT *find_object(char *name);

	FONTCONTAINER *get_font(char *name,int size);

private:
	unsigned int ID_high;
	unsigned int ID_low;

	List<class MOG2_INTERFACE> interfaces;
	List<MAP> maps;
	List<class TILE_SOURCE> tile_sources;
	List<TILE> tiles[TILECATEGORIES];
	List<OBJECT> objects;
	List<class SAMPLE> samples;

	/* GAME: */ 
	List<VARIABLE> game_variables;
	class ActionSCRIPT *game_action;

	/* APPLICATION: */ 
	List<VARIABLE> application_variables;
	int n_app_states;
	class ActionSCRIPT **s_app_action;

	// Text
	List<FONTCONTAINER> fonts;
};

#endif

