#ifdef _WIN32
#include <windows.h>
#include <windowsx.h>
#else
#include <sys/time.h>
#include <time.h>
#endif

#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"
#include "MOG2_auxiliar.h"

#include "BWindow.h"
#include "BActiveWindow.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X;
extern int SCREEN_Y;
extern int COLOUR_DEPTH;
extern bool fullscreen;
extern char *application_name;

/* Redrawing constant: */ 
extern int REDRAWING_PERIOD;
extern SDL_Surface *screen_sfc;
extern int surface_flags;
/*
extern int buffer_fps,act_fps,fps_timer;
extern int buffer_fps_d,fps_d;
*/
BActiveWindow::BActiveWindow(int x,int y,int dx,int dy,TTF_Font *font) : BWindow(x,y,dx,dy,font)
{
} /* BActiveWindow::BActiveWindow */ 


BActiveWindow::~BActiveWindow(void)
{
} /* BActiveWindow::~BActiveWindow */ 



void BActiveWindow::Execute(void)
{
	int time,act_time;
	SDL_Event event;
	bool quit=false;
	SDL_Surface *background_sfc=SDL_CreateRGBSurface(SDL_SRCALPHA|surface_flags,SCREEN_X, SCREEN_Y,COLOUR_DEPTH,0,0,0,0);

	SDL_BlitSurface(screen_sfc,0,background_sfc,0);

	active_window=this;

	time=SDL_GetTicks();
	while (!quit) {
		while( SDL_PollEvent( &event ) ) {
			bool captured=false;

            switch( event.type ) {
                /* Keyboard event */ 
                case SDL_KEYDOWN:
					if (event.key.keysym.sym==SDLK_F12) {
						captured=true;
						quit = true;
					} /* if */ 

					if (event.key.keysym.sym==SDLK_RETURN) {
						SDLMod modifiers=SDL_GetModState();

						if ((modifiers&KMOD_ALT)!=0) {
							captured=true;

							/* Toogle FULLSCREEN mode: */ 
							if (fullscreen) fullscreen=false;
									   else fullscreen=true;
							SDL_QuitSubSystem(SDL_INIT_VIDEO);
							SDL_InitSubSystem(SDL_INIT_VIDEO);
							if (SDL_WasInit(SDL_INIT_VIDEO)) {
								screen_sfc = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0));
								SDL_WM_SetCaption(application_name, 0);
								SDL_ShowCursor(SDL_DISABLE);
								SDL_EnableUNICODE(1);
								dirty=true;
							} else {
								quit = true;
							} /* if */ 
						} /* if */ 
					} /* if */ 
                    break;

                /* SDL_QUIT event (window close) */ 
                case SDL_QUIT:
					captured=true;

                    quit = true;
                    break;
            } /* switch */ 

			if (!captured) {
				if (!this->event(&event)) quit=true;
			} /* if */ 
        } /* while */ 

		act_time=SDL_GetTicks();
		if (act_time-time>=REDRAWING_PERIOD)
		{
/*
			fps++;
			if ((act_time-fps_timer)>1000) {
				fps_timer=act_time;
				buffer_fps=fps;
				buffer_fps_d=fps_d;
				fps=0;
				fps_d=0;
			} // if
*/
			time+=REDRAWING_PERIOD;
			if ((act_time-time)>8*REDRAWING_PERIOD) time=act_time;

			check_mouse();
			if (!cycle(old_x,old_y)) {
				quit=true;
			} else {
//				fps_d++;
				SDL_BlitSurface(background_sfc,0,screen_sfc,0); 
				draw(screen_sfc,old_x,old_y,old_dx,old_dy);

				/* Show mouse: */ 
				draw_mouse_pointer(screen_sfc);

				/* show FPS: */ 
/*
				{
					char tmp[80];
					sprintf(tmp,"FPS: %i/%i",buffer_fps,buffer_fps_d);
					font->print(10,590,tmp,screen_sfc);
				} 
*/
			} /* if */ 
			SDL_Flip(screen_sfc);
		}
	}

	SDL_FreeSurface(background_sfc);
	background_sfc=0;

} /* BActiveWindow::Execute */ 
